/*
	Intermediate model format for game meshes. GameMesh
	www.janvanderweg.com
	jan@janvanderweg.com
	
	- Mesh processor main script
	
	references:
	Lengyel, Eric. Computing Tangent Space Basis Vectors for an Arbitrary Mesh. Terathon Software 3D Graphics Library, 2001. http://www.terathon.com/code/tangent.html
*/ 
filein "gmStructs.ms"
filein "gmHelpers.ms"
filein "gmMaterials.ms"
filein "gmHierarchy.ms"
filein "gmMesh.ms"
filein "gmBuildSurfaces.ms"

struct gmBounds
(
	BMin=[0,0,0],
	BMax=[0,0,0],
	BRadius=0.0
)

struct gmMain
(
	NumNodes=0,
	gmNodes=#(),		-- node containers
	Hierarchy,				-- the bones and bone link hierarchy
	
	NumSurfaces=0,
	Surfaces=#(),
	
	/* Model bounds */
	Bounds,
	
	/* processor settings */
	-- throw an error instead of warning if a node is unsupported
	setting_unsupportednodeerror = false,
	-- throw an error instead of a warning if a node has no/dummy material
	setting_missingmaterialiserror = false,		
	-- group nodes using the same material into a singular surface.
	setting_groupobjectsusingsamematerial = true,	/*	Default is true. Don't change this unless you potentially want a shitload of subsurfaces.
																		Only set to true if you want each node to be a subsurface (or more if submaterials are used) */
	setting_maxweightspervertex = 4, -- Default is 4, don't change unless you know what you're doing. Seriously.
	--- script verbose settings
	setting_verbosehierarchy = false,
	setting_verbosematerial = false,
	
	/* gamemesh internal stuff */
	isError=false,
	whatError="No Error found",
	isAnimated=false,
	GameMeshVersion = 0,
	
	/* GameMesh Initialization stuff don't touch this */
	fn gmInit=
	(
		/* --- GameMeshVersion --- */
		GameMeshVersion = 1
		/* Don't touch the above please */
		
		
		format "/**************************************\\\n"
		format "    GameMesh version: %\n" GameMeshVersion
		format "\\**************************************/\n"
		return true
	),
	initvalue = gmInit(),
	fn GetVersion=(return GameMeshVersion),
	/* Again, don't touch the above please */
	
	fn AddNode node=
	(
		format "--- Adding node to gamemesh ---\n"
		if undefined == node do
		(
			format "Node '%' is undefined.\n" node
			isError = true
			whatError = "AddNode: node undefined\n"
			return false
		)
		/* Addnode sanity checking */
		class = classof node
		if class == Editable_mesh or class == Editable_poly then
		(
			newnode = gmNode()
			newnode.MaxNode = node
			newnode.BMin = node.min
			newnode.BMax = node.max
			newnode.Mesh = (snapshotasmesh node)
			append gmNodes newnode
			NumNodes +=1
			format "\tNode '%' added to nodelist\n" node
		)
		else
		(
			format "\tNode '%' is unsupported.\n" node
			if setting_unsupportednodeerror == true do
			(
				isError = true
				whatError = "AddNode: node unsupported, " + node.name
				return false
			)
		)
	),
	fn Go bBuildSurfaces=
	(
		format "--- Starting GameMesh processor ---\n"
		if NumNodes == 0 do
		(
			isError = true
			whatError = "No nodes added to GameMesh processor"
			return false
		)
		--- Gather some base material data first
		if (gmCollectMaterials this) == false do return false
		--- Construct a hierarchy struct if any of the nodes is animated
		if (gmCollectHierarchy this) == false do return false
		--- Collect mesh info
		if (gmCollectMeshes this) == false do return false
		
		--- Find total min, max and radius
		/* If model is off center bounds will be incorrect. */
		Bounds = gmBounds()
		tmin = [0,0,0] 	--- temp mins, maxs, radiuss
		tmax = [0,0,0]
		tradius=0
		for n = 1 to gmNodes.count do
		(
			nodemin = gmNodes[n].BMin
			nodemax = gmNodes[n].BMax
			noderadius = gmNodes[n].BRadius
			if noderadius > tradius do tradius = noderadius
			if nodemin.x  < tmin.x do tmin.x = nodemin.x
			if nodemin.y  < tmin.y do tmin.y = nodemin.y
			if nodemin.z  < tmin.z do tmin.z = nodemin.z
			if nodemax.x  > tmax.x do tmax.x = nodemax.x
			if nodemax.y  > tmax.y do tmax.y = nodemax.y
			if nodemax.z  > tmax.z do tmax.z = nodemax.z
		)
		Bounds.BRadius = tradius
		Bounds.BMin = tmin
		Bounds.BMax = tmax
		
		format "--- Bounds: ---\n"
		format "Min: %\t\t Max: %\t\t Radius: %\n" Bounds.BMin Bounds.BMax Bounds.BRadius
		
		format "---------------\n"
		format "Nodes processed: %\n" NumNodes
		/*for n = 1 to NumNodes do
			gmNodes[n].DebugOut()*/
		
		if (bBuildSurfaces == true) do
		(
			format "--- Building surfaces ---\n"
			if (gmBuildSurfaces this) == false do return false
			
			/*for s = 1 to Surfaces.count do
				format "%\n" Surfaces[s].Material */
			
			if  setting_groupobjectsusingsamematerial == true do
			(
				gmMergeSurfacesUsingSameMaterial this
			)
			format "**** GAME MESH FINISHED ****\n"
			format "Surfaces: %\n" Surfaces.count
			for s = 1 to Surfaces.count do
			(
				format "V: %\t T: %\t M: %\n" Surfaces[s].NumVerts Surfaces[s].NumTris Surfaces[s].Material
				--Surfaces[s].PrintFullSurface()
			)
		)
	),
	fn DebugOut=
	(
		format "GameMesh debug:\n"
	)
)